@@ -11,14 +11,13 @@ Classes:
  - Feature - Hold the information in a Feature Table.
  - Qualifier - Qualifiers on a Feature.
 
-17-MAR-2009: added support for WGS and WGS_SCAFLD lines.  Ying Huang & Iddo Friedberg
 """
-# local stuff
+
 import Bio.GenBank
 
 
 def _wrapped_genbank(information, indent, wrap_space=1, split_char=" "):
-    """Write a line of GenBank info that can wrap over multiple lines.
+    """Write a line of GenBank info that can wrap over multiple lines (PRIVATE).
 
     This takes a line of information which can potentially wrap over
     multiple lines, and breaks it up with carriage returns and
@@ -46,7 +45,7 @@ def _wrapped_genbank(information, indent, wrap_space=1, split_char=" "):
         cur_pos = 0
         info_parts = []
         while cur_pos < len(information):
-            info_parts.append(information[cur_pos: cur_pos + info_length])
+            info_parts.append(information[cur_pos : cur_pos + info_length])
             cur_pos += info_length
 
     # first get the information string split up by line
@@ -78,7 +77,7 @@ def _wrapped_genbank(information, indent, wrap_space=1, split_char=" "):
 
 
 def _indent_genbank(information, indent):
-    """Write out information with the specified indent.
+    """Write out information with the specified indent (PRIVATE).
 
     Unlike _wrapped_genbank, this function makes no attempt to wrap
     lines -- it assumes that the information already has newlines in the
@@ -96,7 +95,7 @@ def _indent_genbank(information, indent):
     return output_info
 
 
-class Record(object):
+class Record:
     """Hold GenBank information in a format similar to the original record.
 
     The Record class is meant to make data easy to get to when you are
@@ -149,48 +148,57 @@ class Record(object):
     GB_SEQUENCE_INDENT = 9
 
     BASE_FORMAT = "%-" + str(GB_BASE_INDENT) + "s"
-    INTERNAL_FORMAT = " " * GB_INTERNAL_INDENT + "%-" + \
-                      str(GB_BASE_INDENT - GB_INTERNAL_INDENT) + "s"
-    OTHER_INTERNAL_FORMAT = " " * GB_OTHER_INTERNAL_INDENT + "%-" + \
-                            str(GB_BASE_INDENT - GB_OTHER_INTERNAL_INDENT) + \
-                            "s"
+    INTERNAL_FORMAT = (
+        " " * GB_INTERNAL_INDENT + "%-" + str(GB_BASE_INDENT - GB_INTERNAL_INDENT) + "s"
+    )
+    OTHER_INTERNAL_FORMAT = (
+        " " * GB_OTHER_INTERNAL_INDENT
+        + "%-"
+        + str(GB_BASE_INDENT - GB_OTHER_INTERNAL_INDENT)
+        + "s"
+    )
 
     BASE_FEATURE_FORMAT = "%-" + str(GB_FEATURE_INDENT) + "s"
-    INTERNAL_FEATURE_FORMAT = " " * GB_FEATURE_INTERNAL_INDENT + "%-" + \
-                              str(GB_FEATURE_INDENT -
-                                  GB_FEATURE_INTERNAL_INDENT) + "s"
+    INTERNAL_FEATURE_FORMAT = (
+        " " * GB_FEATURE_INTERNAL_INDENT
+        + "%-"
+        + str(GB_FEATURE_INDENT - GB_FEATURE_INTERNAL_INDENT)
+        + "s"
+    )
     SEQUENCE_FORMAT = "%" + str(GB_SEQUENCE_INDENT) + "s"
 
     def __init__(self):
-        """Initialize."""
+        """Initialize the class."""
         self.accession = []
-        self.base_counts = ''
-        self.comment = ''
-        self.contig = ''
-        self.data_file_division = ''
-        self.date = ''
-        self.db_source = ''
+        self.base_counts = ""
+        self.comment = ""
+        self.contig = ""
+        self.data_file_division = ""
+        self.date = ""
+        self.db_source = ""
         self.dblinks = []
-        self.definition = ''
+        self.definition = ""
         self.features = []
-        self.gi = ''
+        self.gi = ""
         self.keywords = []
-        self.locus = ''
-        self.nid = ''
-        self.organism = ''
-        self.origin = ''
-        self.pid = ''
+        self.locus = ""
+        self.molecule_type = ""
+        self.nid = ""
+        self.organism = ""
+        self.origin = ""
+        self.pid = ""
         self.primary = []
         self.projects = []
         self.references = []
-        self.residue_type = ''
-        self.segment = ''
-        self.sequence = ''
-        self.size = ''
-        self.source = ''
+        self.residue_type = ""
+        self.segment = ""
+        self.sequence = ""
+        self.size = ""
+        self.source = ""
         self.taxonomy = []
-        self.version = ''
-        self.wgs = ''
+        self.topology = ""
+        self.version = ""
+        self.wgs = ""
         self.wgs_scafld = []
 
     def __str__(self):
@@ -235,7 +243,7 @@ class Record(object):
         return output
 
     def _locus_line(self):
-        """Provide the output string for the LOCUS line."""
+        """Provide the output string for the LOCUS line (PRIVATE)."""
         output = "LOCUS"
         output += " " * 7  # 6-12 spaces
         output += "%-9s" % self.locus
@@ -267,13 +275,13 @@ class Record(object):
         return output
 
     def _definition_line(self):
-        """Provide output for the DEFINITION line."""
+        """Provide output for the DEFINITION line (PRIVATE)."""
         output = Record.BASE_FORMAT % "DEFINITION"
         output += _wrapped_genbank(self.definition + ".", Record.GB_BASE_INDENT)
         return output
 
     def _accession_line(self):
-        """Output for the ACCESSION line."""
+        """Output for the ACCESSION line (PRIVATE)."""
         if self.accession:
             output = Record.BASE_FORMAT % "ACCESSION"
 
@@ -289,7 +297,7 @@ class Record(object):
         return output
 
     def _version_line(self):
-        """Output for the VERSION line."""
+        """Output for the VERSION line (PRIVATE)."""
         if self.version:
             output = Record.BASE_FORMAT % "VERSION"
             output += self.version
@@ -315,7 +323,7 @@ class Record(object):
         return output
 
     def _nid_line(self):
-        """Output for the NID line. Use of NID is obsolete in GenBank files."""
+        """Output for the NID line. Use of NID is obsolete in GenBank files (PRIVATE)."""
         if self.nid:
             output = Record.BASE_FORMAT % "NID"
             output += "%s\n" % self.nid
@@ -324,7 +332,7 @@ class Record(object):
         return output
 
     def _pid_line(self):
-        """Output for PID line. Presumedly, PID usage is also obsolete."""
+        """Output for PID line. Presumedly, PID usage is also obsolete (PRIVATE)."""
         if self.pid:
             output = Record.BASE_FORMAT % "PID"
             output += "%s\n" % self.pid
@@ -333,9 +341,9 @@ class Record(object):
         return output
 
     def _keywords_line(self):
-        """Output for the KEYWORDS line."""
+        """Output for the KEYWORDS line (PRIVATE)."""
         output = ""
-        if len(self.keywords) >= 0:
+        if self.keywords:
             output += Record.BASE_FORMAT % "KEYWORDS"
             keyword_info = ""
             for keyword in self.keywords:
@@ -344,13 +352,12 @@ class Record(object):
             keyword_info = keyword_info[:-2]
             keyword_info += "."
 
-            output += _wrapped_genbank(keyword_info,
-                                       Record.GB_BASE_INDENT)
+            output += _wrapped_genbank(keyword_info, Record.GB_BASE_INDENT)
 
         return output
 
     def _db_source_line(self):
-        """Output for DBSOURCE line."""
+        """Output for DBSOURCE line (PRIVATE)."""
         if self.db_source:
             output = Record.BASE_FORMAT % "DBSOURCE"
             output += "%s\n" % self.db_source
@@ -359,7 +366,7 @@ class Record(object):
         return output
 
     def _segment_line(self):
-        """Output for the SEGMENT line."""
+        """Output for the SEGMENT line (PRIVATE)."""
         output = ""
         if self.segment:
             output += Record.BASE_FORMAT % "SEGMENT"
@@ -367,13 +374,13 @@ class Record(object):
         return output
 
     def _source_line(self):
-        """Output for SOURCE line on where the sample came from."""
+        """Output for SOURCE line on where the sample came from (PRIVATE)."""
         output = Record.BASE_FORMAT % "SOURCE"
         output += _wrapped_genbank(self.source, Record.GB_BASE_INDENT)
         return output
 
     def _organism_line(self):
-        """Output for ORGANISM line with taxonomy info."""
+        """Output for ORGANISM line with taxonomy info (PRIVATE)."""
         output = Record.INTERNAL_FORMAT % "ORGANISM"
         # Now that species names can be too long, this line can wrap (Bug 2591)
         output += _wrapped_genbank(self.organism, Record.GB_BASE_INDENT)
@@ -389,16 +396,15 @@ class Record(object):
         return output
 
     def _comment_line(self):
-        """Output for the COMMENT lines."""
+        """Output for the COMMENT lines (PRIVATE)."""
         output = ""
         if self.comment:
             output += Record.BASE_FORMAT % "COMMENT"
-            output += _indent_genbank(self.comment,
-                                      Record.GB_BASE_INDENT)
+            output += _indent_genbank(self.comment, Record.GB_BASE_INDENT)
         return output
 
     def _features_line(self):
-        """Output for the FEATURES line."""
+        """Output for the FEATURES line (PRIVATE)."""
         output = ""
         if len(self.features) > 0:
             output += Record.BASE_FEATURE_FORMAT % "FEATURES"
@@ -406,14 +412,14 @@ class Record(object):
         return output
 
     def _base_count_line(self):
-        """Output for the BASE COUNT line with base information."""
+        """Output for the BASE COUNT line with base information (PRIVATE)."""
         output = ""
         if self.base_counts:
             output += Record.BASE_FORMAT % "BASE COUNT  "
             # split up the base counts into their individual parts
             count_parts = self.base_counts.split(" ")
-            while '' in count_parts:
-                count_parts.remove('')
+            while "" in count_parts:
+                count_parts.remove("")
             # deal with the standard case, with a normal origin line
             # like: 474 a    356 c    428 g    364 t
             if len(count_parts) % 2 == 0:
@@ -431,20 +437,19 @@ class Record(object):
         return output
 
     def _origin_line(self):
-        """Output for the ORIGIN line."""
+        """Output for the ORIGIN line (PRIVATE)."""
         output = ""
         # only output the ORIGIN line if we have a sequence
         if self.sequence:
             output += Record.BASE_FORMAT % "ORIGIN"
             if self.origin:
-                output += _wrapped_genbank(self.origin,
-                                           Record.GB_BASE_INDENT)
+                output += _wrapped_genbank(self.origin, Record.GB_BASE_INDENT)
             else:
                 output += "\n"
         return output
 
     def _sequence_line(self):
-        """Output for all of the sequence."""
+        """Output for all of the sequence (PRIVATE)."""
         output = ""
         if self.sequence:
             cur_seq_pos = 0
@@ -466,30 +471,31 @@ class Record(object):
         return output
 
     def _wgs_line(self):
-            output = ""
-            if self.wgs:
-                    output += Record.BASE_FORMAT % "WGS"
-                    output += self.wgs
-            return output
+        output = ""
+        if self.wgs:
+            output += Record.BASE_FORMAT % "WGS"
+            output += self.wgs
+        return output
 
     def _wgs_scafld_line(self):
-            output = ""
-            if self.wgs_scafld:
-                    output += Record.BASE_FORMAT % "WGS_SCAFLD"
-                    output += self.wgs_scafld
-            return output
+        output = ""
+        if self.wgs_scafld:
+            output += Record.BASE_FORMAT % "WGS_SCAFLD"
+            output += self.wgs_scafld
+        return output
 
     def _contig_line(self):
-        """Output for CONTIG location information from RefSeq."""
+        """Output for CONTIG location information from RefSeq (PRIVATE)."""
         output = ""
         if self.contig:
             output += Record.BASE_FORMAT % "CONTIG"
-            output += _wrapped_genbank(self.contig,
-                                       Record.GB_BASE_INDENT, split_char=',')
+            output += _wrapped_genbank(
+                self.contig, Record.GB_BASE_INDENT, split_char=","
+            )
         return output
 
 
-class Reference(object):
+class Reference:
     """Hold information from a GenBank reference.
 
     Attributes:
@@ -506,16 +512,16 @@ class Reference(object):
     """
 
     def __init__(self):
-        """Initialize."""
-        self.number = ''
-        self.bases = ''
-        self.authors = ''
-        self.consrtm = ''
-        self.title = ''
-        self.journal = ''
-        self.medline_id = ''
-        self.pubmed_id = ''
-        self.remark = ''
+        """Initialize the class."""
+        self.number = ""
+        self.bases = ""
+        self.authors = ""
+        self.consrtm = ""
+        self.title = ""
+        self.journal = ""
+        self.medline_id = ""
+        self.pubmed_id = ""
+        self.remark = ""
 
     def __str__(self):
         """Convert the reference to a GenBank format string."""
@@ -531,7 +537,7 @@ class Reference(object):
         return output
 
     def _reference_line(self):
-        """Output for REFERENCE lines."""
+        """Output for REFERENCE lines (PRIVATE)."""
         output = Record.BASE_FORMAT % "REFERENCE"
         if self.number:
             if self.bases:
@@ -544,7 +550,7 @@ class Reference(object):
         return output
 
     def _authors_line(self):
-        """Output for AUTHORS information."""
+        """Output for AUTHORS information (PRIVATE)."""
         output = ""
         if self.authors:
             output += Record.INTERNAL_FORMAT % "AUTHORS"
@@ -552,7 +558,7 @@ class Reference(object):
         return output
 
     def _consrtm_line(self):
-        """Output for CONSRTM information."""
+        """Output for CONSRTM information (PRIVATE)."""
         output = ""
         if self.consrtm:
             output += Record.INTERNAL_FORMAT % "CONSRTM"
@@ -560,7 +566,7 @@ class Reference(object):
         return output
 
     def _title_line(self):
-        """Output for TITLE information."""
+        """Output for TITLE information (PRIVATE)."""
         output = ""
         if self.title:
             output += Record.INTERNAL_FORMAT % "TITLE"
@@ -568,7 +574,7 @@ class Reference(object):
         return output
 
     def _journal_line(self):
-        """Output for JOURNAL information."""
+        """Output for JOURNAL information (PRIVATE)."""
         output = ""
         if self.journal:
             output += Record.INTERNAL_FORMAT % "JOURNAL"
@@ -576,7 +582,7 @@ class Reference(object):
         return output
 
     def _medline_line(self):
-        """Output for MEDLINE information."""
+        """Output for MEDLINE information (PRIVATE)."""
         output = ""
         if self.medline_id:
             output += Record.INTERNAL_FORMAT % "MEDLINE"
@@ -584,7 +590,7 @@ class Reference(object):
         return output
 
     def _pubmed_line(self):
-        """Output for PUBMED information."""
+        """Output for PUBMED information (PRIVATE)."""
         output = ""
         if self.pubmed_id:
             output += Record.OTHER_INTERNAL_FORMAT % "PUBMED"
@@ -592,7 +598,7 @@ class Reference(object):
         return output
 
     def _remark_line(self):
-        """Output for REMARK information."""
+        """Output for REMARK information (PRIVATE)."""
         output = ""
         if self.remark:
             output += Record.INTERNAL_FORMAT % "REMARK"
@@ -600,43 +606,38 @@ class Reference(object):
         return output
 
 
-class Feature(object):
+class Feature:
     """Hold information about a Feature in the Feature Table of GenBank record.
 
     Attributes:
      - key - The key name of the featue (ie. source)
      - location - The string specifying the location of the feature.
-     - qualfiers - A listing Qualifier objects in the feature.
+     - qualfiers - A list of Qualifier objects in the feature.
 
     """
 
-    def __init__(self):
-        """Initialize."""
-        self.key = ''
-        self.location = ''
+    def __init__(self, key="", location=""):
+        """Initialize the class."""
+        self.key = key
+        self.location = location
         self.qualifiers = []
 
+    def __repr__(self):
+        """Representation of the object for debugging or logging."""
+        return "Feature(key=%r, location=%r)" % (self.key, self.location)
+
     def __str__(self):
         """Return feature as a GenBank format string."""
         output = Record.INTERNAL_FEATURE_FORMAT % self.key
-        output += _wrapped_genbank(self.location, Record.GB_FEATURE_INDENT,
-                                   split_char=',')
+        output += _wrapped_genbank(
+            self.location, Record.GB_FEATURE_INDENT, split_char=","
+        )
         for qualifier in self.qualifiers:
-            output += " " * Record.GB_FEATURE_INDENT
-
-            # determine whether we can wrap on spaces
-            space_wrap = 1
-            for no_space_key in \
-                    Bio.GenBank._BaseGenBankConsumer.remove_space_keys:
-                if no_space_key in qualifier.key:
-                    space_wrap = 0
-
-            output += _wrapped_genbank(qualifier.key + qualifier.value,
-                                       Record.GB_FEATURE_INDENT, space_wrap)
+            output += str(qualifier)
         return output
 
 
-class Qualifier(object):
+class Qualifier:
     """Hold information about a qualifier in a GenBank feature.
 
     Attributes:
@@ -645,7 +646,24 @@ class Qualifier(object):
 
     """
 
-    def __init__(self):
-        """Initialize."""
-        self.key = ''
-        self.value = ''
+    def __init__(self, key="", value=""):
+        """Initialize the class."""
+        self.key = key
+        self.value = value
+
+    def __repr__(self):
+        """Representation of the object for debugging or logging."""
+        return "Qualifier(key=%r, value=%r)" % (self.key, self.value)
+
+    def __str__(self):
+        """Return feature qualifier as a GenBank format string."""
+        output = " " * Record.GB_FEATURE_INDENT
+        # determine whether we can wrap on spaces
+        space_wrap = 1
+        for no_space_key in Bio.GenBank._BaseGenBankConsumer.remove_space_keys:
+            if no_space_key in self.key:
+                space_wrap = 0
+        # return double quotes as-is, leave it to the user to escape them
+        return output + _wrapped_genbank(
+            self.key + self.value, Record.GB_FEATURE_INDENT, space_wrap
+        )
