@@ -7,6 +7,7 @@
 
 import re
 from functools import reduce
+from abc import ABC, abstractmethod
 
 from Bio.SearchIO._index import SearchIndexer
 from Bio.SearchIO._model import QueryResult, Hit, HSP, HSPFragment
@@ -200,10 +201,10 @@ def _split_fragment(frag):
         # coordinates for the split strand
         qstart, hstart = qpos, hpos
         qpos += (
-            len(split) - sum(str(split.query.seq).count(x) for x in ("-", "<", ">"))
+            len(split) - sum(split.query.seq.count(x) for x in ("-", "<", ">"))
         ) * qstep
         hpos += (
-            len(split) - sum(str(split.hit.seq).count(x) for x in ("-", "<", ">"))
+            len(split) - sum(split.hit.seq.count(x) for x in ("-", "<", ">"))
         ) * hstep
 
         split.hit_start = min(hstart, hpos)
@@ -214,7 +215,7 @@ def _split_fragment(frag):
         # account for frameshift length
         abs_slice = slice(abs_pos + s_start, abs_pos + s_stop)
         if len(frag.aln_annotation) == 2:
-            seqs = (str(frag[abs_slice].query.seq), str(frag[abs_slice].hit.seq))
+            seqs = (frag[abs_slice].query.seq, frag[abs_slice].hit.seq)
         elif len(frag.aln_annotation) == 3:
             seqs = (
                 frag[abs_slice].aln_annotation["query_annotation"],
@@ -291,7 +292,7 @@ def _create_hsp(hid, qid, hspd):
         "model",
         "vulgar_comp",
         "cigar_comp",
-        "alphabet",
+        "molecule_type",
     ):
         if attr in hspd:
             setattr(hsp, attr, hspd[attr])
@@ -310,8 +311,8 @@ def _parse_hit_or_query_line(line):
     return id, desc
 
 
-class _BaseExonerateParser:
-    """Abstract iterator for exonerate format."""
+class _BaseExonerateParser(ABC):
+    """Abstract base class iterator for exonerate format."""
 
     _ALN_MARK = None
 
@@ -351,8 +352,9 @@ class _BaseExonerateParser:
             else:
                 self.line = self.handle.readline()
 
+    @abstractmethod
     def parse_alignment_block(self, header):
-        raise NotImplementedError("Subclass must implement this")
+        raise NotImplementedError
 
     def _parse_alignment_header(self):
         # read all header lines and store them
